#!/bin/ksh

#**************************************************************
# Author: Sbastien Denvil
# Contact: Sebastien.Denvil__at__ipsl.jussieu.fr
# $Revision::                                          $ Revision of last commit
# $Author::                                            $ Author of last commit
# $Date::                                              $ Date of last commit
# IPSL (2006)
#  This software is governed by the CeCILL licence see libIGCM/libIGCM_CeCILL.LIC
#
#**************************************************************

#=========================================================
# The documentation of this file can be automatically generated
# if you use the prefix #D- for comments to be extracted.
# Extract with command: cat lib* | grep "^#D-" | cut -c "4-"
#=========================================================

#D-#==================================================
#D-LibIGCM_sys for Vargas
#D-#==================================================
#D-
#D- This ksh library if a layer under some usefull
#D-environment variables and shell commands.
#D-All those definitions depend on host particularities.
#D-It manages a stack mechanism and test validity of operations.
#D-All function described bellow must be prefixed by IGCM_sys.

#====================================================
# libIGCM_sys PARAMETERS
#====================================================

#====================================================
# set DEBUG_sys to true to output calls of function
typeset -r DEBUG_sys=${DEBUG_sys:=true}

#====================================================
# Turn in dry run mode ? (sys_Put_Rest, sys_Put_Out, sys_Get)
typeset -r DRYRUN=${DRYRUN:=0}

# YOU MUST COMPILE YOUR EXE FILES FOR DRYRUN MODE !
# -------------------------------------------------------------------------------------
# | DRYRUN=  |  Date computations, | sys_Get    |  Exe    | sys_Put_Out; sys_Put_Rest |
# |          |  Cp/Exe param files |            |  Chmod  |                           |
# |          |      Qsub           |            |         |                           |
# -------------------------------------------------------------------------------------
# |    0     |       yes           |    yes     |  yes    |      yes                  |
# -------------------------------------------------------------------------------------
# |    1     |       yes           |    yes     |  yes    |      no                   |
# -------------------------------------------------------------------------------------
# |    2     |       yes           |    yes     |  no     |      no                   |
# -------------------------------------------------------------------------------------
# |    3     |       yes           |    no      |  no     |      no                   |
# -------------------------------------------------------------------------------------

#=====================================================
# Global Variables :
#=====================================================
# Language : "fr" or "en"
typeset -r MYLANG="fr"

#=====================================================
# Host and user names
# $hostname ou hostname
typeset  HOST=${HOST:=$( hostname )}
# $username ou whoami
typeset  LOGIN=${LOGIN:=$( whoami )}
# $hostname of the MASTER job
typeset -r MASTER=vargas

#D-
#D-#==================================================
#D-Program used in libIGCM
#D-#==================================================

# rsync with path
typeset -r RSYNC=/usr/local/bin/rsync
# RSYNC_opt args to rsync
typeset -r RSYNC_opt="-va"
# ie storage filesystem
typeset -r STOREHOST=gaya.idris.fr
typeset -r REMOTE_RSYNC=/u/rech/ces/rces452/RSYNC/bin/rsync

#====================================================
# Host specific DIRECTORIES
#====================================================

#====================================================
#- R_EXE   (==> BIN_DIR = ${MODIPSL}/bin )
typeset -r R_EXE="${MODIPSL}/bin"

#====================================================
#- SUBMIT_DIR : submission dir
typeset SUBMIT_DIR=${SUBMIT_DIR:=${LOADL_STEP_INITDIR}}

#====================================================
#- ARCHIVE
typeset -r ARCHIVE=$( echo ${HOME} | sed -e "s/homegpfs/u/" )

#====================================================
#- Mirror libIGCM from vargas to ulam
typeset -r MirrorlibIGCM=${MirrorlibIGCM:=true}

#====================================================
#- libIGCM_POST
PATHlibIGCM=$( echo ${libIGCM} | gawk -F"${LOGIN}/" '{print $2}' | sed -e "s&/libIGCM&&" )
typeset -r HOME_POST=$( echo ${HOME} | sed -e "s/homegpfs/home/" )
typeset -r libIGCM_POST=${HOME_POST}/MIRROR/${PATHlibIGCM}/libIGCM

#====================================================
#- IN
typeset -r R_IN=${R_IN:=/u/rech/psl/rpsl035/IGCM}
typeset -r R_IN_ECMWF=${R_IN_ECMWF:=/u/rech/psl/rpsl376}

#====================================================
#- OUT
typeset -r R_OUT=${ARCHIVE}/IGCM_OUT

#====================================================
#- R_BUF  Buffer to pack files
typeset -r R_BUF=${WORKDIR}/IGCM_OUT

#====================================================
#- OUT_POST
typeset -r R_OUT_POST=$( echo ${HOME} | sed -e "s/homegpfs/workdir/" )/IGCM_OUT

#====================================================
#- RUN_DIR_PATH : Temporary working directory (=> TMP)
typeset -r RUN_DIR_PATH=${RUN_DIR_PATH:=${TMPDIR}}

#====================================================
#- BIG_DIR : BIG_DIR to store files waiting for rebuild
typeset -r BIG_DIR=${BIG_DIR:=${WORKDIR}/REBUILD}

#====================================================
#- HOST_MPIRUN_COMMAND
typeset -r HOST_MPIRUN_COMMAND=${HOST_MPIRUN_COMMAND:="time poe"}

#====================================================
#- Max number of arguments passed to nco operator or demigration command
UNIX_MAX_LIMIT=120

#====================================================
#- Default number of MPI task for IPSL coupled model
#- required for backward compatibility
#-
DEFAULT_NUM_PROC_OCE=5
DEFAULT_NUM_PROC_CPL=1
[ X${BATCH_NUM_PROC_TOT} = X ] || (( DEFAULT_NUM_PROC_ATM = BATCH_NUM_PROC_TOT - DEFAULT_NUM_PROC_OCE - DEFAULT_NUM_PROC_CPL ))
DEFAULT_NUM_PROC_TOTAL=${BATCH_NUM_PROC_TOT}

#D-#==================================================
#D-function IGCM_sys_RshMaster
#D-* Purpose: Connection to frontend machine.
#D-* Examples:
#D-
(( RshMNum = 0 ))
function IGCM_sys_RshMaster {
  IGCM_debug_PushStack "IGCM_sys_RshMaster" $@

  typeset THEFILE OUTFILE ERR

  RshDIR=${WORKDIR}/RUN_DIR/${config_UserChoices_TagName}/${config_UserChoices_JobName}.${$}
  if [ ! -d ${RshDIR} ] ; then
    IGCM_sys_Mkdir ${RshDIR}
  fi
  THEFILE=${RshDIR}/file_sys_RshMaster_$$_${RshMNum}
  echo '#!/bin/ksh' > ${THEFILE}
  echo "export libIGCM=${libIGCM} > /dev/null 2>&1" >> ${THEFILE}
  echo "export DEBUG_debug=${DEBUG_debug} > /dev/null 2>&1" >> ${THEFILE}
  echo ". ${libIGCM}/libIGCM_debug/libIGCM_debug.ksh > /dev/null 2>&1" >> ${THEFILE}
  echo ". ${libIGCM}/libIGCM_card/libIGCM_card.ksh > /dev/null 2>&1" >> ${THEFILE}
  echo "${@} " >> ${THEFILE}

  # No frontend queue on brodie : just rsh the job on the frontend with submission.
  sed -e "s@IGCM_sys_Qsub\(.*\)@\1@" ${THEFILE} > ${THEFILE}_

  chmod u+x  ${THEFILE}_

  OUTFILE=${RshDIR}/out_RshMaster.$$.${RshMNum}
  (( RshMNum = RshMNum + 1 ))

  rsh ${MASTER} 'bash -c "nohup '${THEFILE}'_ > '${OUTFILE}' 2>&1 &"'
  if [ $? -gt 0 ] ; then
    echo "IGCM_sys_RshMaster : erreur."
    IGCM_debug_Exit "IGCM_sys_RshMaster"
  fi
  IGCM_debug_PopStack "IGCM_sys_RshMaster"
}

#D-#==================================================
#D-function IGCM_sys_RshArchive
#D-* Purpose: Archive rsh command
#D-* Examples:
#D-
function IGCM_sys_RshArchive {
  IGCM_debug_PushStack "IGCM_sys_RshArchive" $@
  rsh ${STOREHOST} exec /bin/ksh <<-EOF
    ${@}
EOF
  if [ $? -gt 0 ] ; then
    echo "IGCM_sys_RshArchive : erreur."
    IGCM_debug_Exit "IGCM_sys_RshArchive"
  fi
  IGCM_debug_PopStack "IGCM_sys_RshArchive"
}

#D-#==================================================
#D-function IGCM_sys_RshPost
#D-* Purpose: Post-process rsh command
#D-* Examples:
#D-
function IGCM_sys_RshPost {
  IGCM_debug_PushStack "IGCM_sys_RshPost" $@
  if ( $DEBUG_sys ) ; then
    echo "IGCM_sys_RshPost :" $@
  fi

  # keep standard input to keep it for postpone if ulam don't answer
  cat >/tmp/tmp_IGCM_sys_RshPost_$$_${LOGNAME}

  ## Add submission for old postponed files only if command is a Qsub
  ##
  ISITQSUB=0
  grep Qsub /tmp/tmp_IGCM_sys_RshPost_$$_${LOGNAME} >/dev/null 2>&1 && ISITQSUB=1
  if  ( [ ${ISITQSUB} -gt 0 ] ) ; then
    if [ -d ${SUBMIT_DIR}/POSTPONE ] ; then
      for postponed_file in ${SUBMIT_DIR}/POSTPONE/* ; do
        if [ -f ${postponed_file} ] ; then
          IGCM_debug_Print 2 " submit postponed file : " ${postponed_file}
          rsh ulam exec /bin/ksh <${postponed_file}
          if [ $? -eq 0 ] ; then
            IGCM_sys_Rm ${postponed_file}
          else
            IGCM_debug_Print 2 "keep postponed file for an other submission :" ${postponed_file}
          fi
        fi
      done
    fi
  fi

  RETURN=0
  # rsh ulam command
  rsh ulam exec /bin/ksh </tmp/tmp_IGCM_sys_RshPost_$$_${LOGNAME}
  RETURN=$?

  # keep standard input for postpone if ulam don't answer
  if ( [ ${RETURN} -gt 0 ] ) ; then
    IGCM_debug_Print 2 "Postpone tmp_IGCM_sys_RshPost_$$_${LOGNAME}"
    [ -d ${SUBMIT_DIR}/POSTPONE ] || mkdir ${SUBMIT_DIR}/POSTPONE
    mv /tmp/tmp_IGCM_sys_RshPost_$$_${LOGNAME} ${SUBMIT_DIR}/POSTPONE
  else
    IGCM_sys_Rm /tmp/tmp_IGCM_sys_RshPost_$$_${LOGNAME}
  fi
  IGCM_debug_PopStack "IGCM_sys_RshPost"
}

#D-#==================================================
#D-function IGCM_sys_SendMail
#D-* Purpose: Send mail when simulation is over
#D-* Examples:
#D-
function IGCM_sys_SendMail {
  IGCM_debug_PushStack "IGCM_sys_SendMail" $@
  if ( $DEBUG_sys ) ; then
    echo "IGCM_sys_SendMail :" $@
  fi

  if ( ${ExitFlag} ) ; then
    status=failed
  else
    status=completed
  fi
  cat  << END_MAIL > job_end.mail
Dear ${LOGIN},

  Simulation ${config_UserChoices_JobName} is ${status} on supercomputer `hostname`.
  Job started : ${DateBegin}
  Job ended   : ${DateEnd}
  Output files are available in ${R_SAVE}
  Script files, Script Outputs and Debug files (if necessary) are available in ${SUBMIT_DIR}
END_MAIL

  if [ ! -z ${config_UserChoices_MailName} ] ; then
    mailx -s "${config_UserChoices_JobName} ${status}" ${config_UserChoices_MailName} <  job_end.mail
  elif [ -f ~/.forward ] ; then
    mailx -s "${config_UserChoices_JobName} ${status}" $( cat ~/.forward ) < job_end.mail
  else
    mailx -s "${config_UserChoices_JobName} ${status}" ${USER} < job_end.mail
  fi

  if [ $? -gt 0 ] ; then
    echo "IGCM_sys_SendMail : erreur."
    IGCM_debug_Exit "IGCM_sys_SendMail"
  fi
  IGCM_debug_PopStack "IGCM_sys_SendMail"
}

#D-#==================================================
#D-function IGCM_sys_Mkdir
#D-* Purpose: Master locale mkdir command
#D-* Examples:
#D-
function IGCM_sys_Mkdir {
  IGCM_debug_PushStack "IGCM_sys_Mkdir" $@
  if ( $DEBUG_sys ) ; then
    echo "IGCM_sys_Mkdir :" $@
  fi
  if [ ! -d ${1} ]; then
    \mkdir -p $1
    if [ $? -gt 0 ] ; then
      echo "IGCM_sys_Mkdir : erreur."
      IGCM_debug_Exit "IGCM_sys_Mkdir"
    fi
  fi
    # vrification :
  if [ ! -d ${1} ] ; then
    echo "IGCM_sys_Mkdir : erreur."
    IGCM_debug_Exit "IGCM_sys_Mkdir"
  fi
  IGCM_debug_PopStack "IGCM_sys_Mkdir"
}

#D-#==================================================
#D-function IGCM_sys_MkdirArchive
#D-* Purpose: Mkdir on Archive
#D-* Examples:
#D-
function IGCM_sys_MkdirArchive {
  IGCM_debug_PushStack "IGCM_sys_MkdirArchive" $@
  if ( $DEBUG_sys ) ; then
    echo "IGCM_sys_MkdirArchive :" $@
  fi
  #- creation de repertoire sur le serveur fichier
  rsh ${STOREHOST} -n mkdir -p $1

  if [ $? -gt 0 ] ; then
    echo "IGCM_sys_MkdirArchive : erreur."
    IGCM_debug_Exit "IGCM_sys_MkdirArchive"
  fi
  IGCM_debug_PopStack "IGCM_sys_MkdirArchive"
}

#D-#==================================================
#D-function IGCM_sys_MkdirWork
#D-* Purpose: Mkdir on Work
#D-* Examples:
#D-
function IGCM_sys_MkdirWork {
  IGCM_debug_PushStack "IGCM_sys_MkdirWork" $@
  if ( $DEBUG_sys ) ; then
    echo "IGCM_sys_MkdirWork :" $@
  fi
  #- creation de repertoire sur le serveur fichier
  if [ ! -d ${1} ]; then
    \mkdir -p $1
    if [ $? -gt 0 ] ; then
      echo "IGCM_sys_MkdirWork : erreur."
      IGCM_debug_Exit "IGCM_sys_MkdirWork"
    fi
  fi
  IGCM_debug_PopStack "IGCM_sys_MkdirWork"
}

#D-#==================================================
#D-function IGCM_sys_Cd
#D-* Purpose: master cd command
#D-* Examples:
#D-
function IGCM_sys_Cd {
  IGCM_debug_PushStack "IGCM_sys_Cd" $@
  if ( $DEBUG_sys ) ; then
    echo "IGCM_sys_Cd :" $@
  fi
  \cd $1
  if [ $? -gt 0 ] ; then
    echo "IGCM_sys_Cd : erreur."
    IGCM_debug_Exit "IGCM_sys_Cd"
  fi
  IGCM_debug_PopStack "IGCM_sys_Cd"
}

#D-#==================================================
#D-function IGCM_sys_Chmod
#D-* Purpose: Chmod
#D-* Examples:
#D-
function IGCM_sys_Chmod {
  IGCM_debug_PushStack "IGCM_sys_Chmod" -- $@
  if ( $DEBUG_sys ) ; then
    echo "IGCM_sys_Chmod :" $@
  fi
  if [ $DRYRUN -le 1 ]; then
    \chmod $@
    if [ $? -gt 0 ] ; then
      echo "IGCM_sys_Chmod : erreur."
      IGCM_debug_Exit "IGCM_sys_Chmod"
    fi
  else
    ( ${DEBUG_debug} ) && echo "DRYRUN mode = " $DRYRUN >> stack
  fi
  IGCM_debug_PopStack "IGCM_sys_Chmod"
}

#D-#==================================================
#D-function IGCM_sys_FileSize
#D-* Purpose: Filesize
#D-* Examples:
#D-
function IGCM_sys_FileSize {
  IGCM_debug_PushStack "IGCM_sys_FileSize" $@

  typeset sizeF
  set +A sizeF -- $( ls -la ${1} )
  if [ $? -gt 0 ] ; then
    IGCM_debug_Exit "IGCM_sys_FileSize"
  fi
  eval ${2}=${sizeF[4]}

  IGCM_debug_PopStack "IGCM_sys_FileSize"
}

#D-#==================================================
#D-function IGCM_sys_TestDir
#D-* Purpose: Test Directory that must exists
#D-* Examples:
#D-
function IGCM_sys_TestDir {
  IGCM_debug_PushStack "IGCM_sys_TestDir" $@
  if ( $DEBUG_sys ) ; then
    echo "IGCM_sys_TestDir :" $@
  fi
  typeset ExistFlag
  ExistFlag=$( [ -d $1 ] && echo 0 || echo 1 )
  IGCM_debug_PopStack "IGCM_sys_TestDir"

  return ${ExistFlag}
}

#D-#==================================================
#D-function IGCM_sys_TestDirArchive
#D-* Purpose: Test Directory that must exists on Archive
#D-* Examples:
#D-
function IGCM_sys_TestDirArchive {
  IGCM_debug_PushStack "IGCM_sys_TestDirArchive" $@
  if ( $DEBUG_sys ) ; then
    echo "IGCM_sys_TestDirArchive :" $@
  fi
  typeset ExistFlag
  ExistFlag=$( IGCM_sys_RshArchive "[ -d $1 ] && echo 0 || echo 1" )
  IGCM_debug_PopStack "IGCM_sys_TestDirArchive"

  return ${ExistFlag}
}

#D-#==================================================
#D-function IGCM_sys_IsFileArchived
#D-* Purpose: Test file that must NOT EXISTS on Archive
#D-* always true on vargas.
#D-* Examples:
#D-
function IGCM_sys_IsFileArchived {
  IGCM_debug_PushStack "IGCM_sys_IsFileArchived" $@
  if ( $DEBUG_sys ) ; then
    echo "IGCM_sys_IsFileArchived :" $@
  fi
  typeset IsArchivedFlag
  IsArchivedFlag=$( echo 0 )
  IGCM_debug_PopStack "IGCM_sys_IsFileArchived"

  return ${IsArchivedFlag}
}

#D-#==================================================
#D-function IGCM_sys_TestFileArchive
#D-* Purpose: Test file that must NOT EXISTS on Archive
#D-* Examples:
#D-
function IGCM_sys_TestFileArchive {
  IGCM_debug_PushStack "IGCM_sys_TestFileArchive" $@
  typeset ExistFlag
  ExistFlag=$( IGCM_sys_RshArchive "[ -f $1 ] && echo 0 || echo 1" )
  IGCM_debug_PopStack "IGCM_sys_TestFileArchive"

  return ${ExistFlag}
}

#D-#==================================================
#D-function IGCM_sys_TestFileBuffer
#D-* Purpose: Test file that must NOT EXISTS on Buffer
#D-* Examples:
#D-
function IGCM_sys_TestFileBuffer {
  IGCM_debug_PushStack "IGCM_sys_TestFileBuffer" $@
  typeset ExistFlag
  ExistFlag=$( [ -f $1 ] && echo 0 || echo 1 )
  IGCM_debug_PopStack "IGCM_sys_TestFileBuffer"

  return ${ExistFlag}
}

#D-#==================================================
#D-function IGCM_sys_CountFileArchive
#D-* Purpose: Count files on Archive filesystem
#D-* Examples:
#D-
function IGCM_sys_CountFileArchive {
  IGCM_debug_PushStack "IGCM_sys_CountFileArchive" $@
  IGCM_sys_RshArchive "ls ${@} 2>/dev/null | wc -l"
  if [ $? -gt 0 ] ; then
    echo "IGCM_sys_CountFileArchive : erreur."
  fi
  IGCM_debug_PopStack "IGCM_sys_CountFileArchive"
}

#D-#==================================================
#D-function IGCM_sys_CountFileBuffer
#D-* Purpose: Count files on Scratch filesystem
#D-* Examples:
#D-
function IGCM_sys_CountFileBuffer {
  IGCM_debug_PushStack "IGCM_sys_CountFileBuffer" $@
  ls ${@} 2>/dev/null | wc -l
  if [ $? -gt 0 ] ; then
    echo "IGCM_sys_CountFileBuffer : erreur."
  fi
  IGCM_debug_PopStack "IGCM_sys_CountFileBuffer"
}

#D-#==================================================
#D-function IGCM_sys_Tree
#D-* Purpose: Tree directories with files on ${ARCHIVE}
#D-* Examples: IGCM_sys_Tree ${R_IN} ${R_OUT}
#D-
function IGCM_sys_Tree {
  IGCM_debug_PushStack "IGCM_sys_Tree" $@
  if ( $DEBUG_sys ) ; then
    echo "IGCM_sys_Tree :" $@
  fi

  \mfls -r $@

  IGCM_debug_PopStack "IGCM_sys_Tree"
}

#D-#==================================================
#D-function IGCM_sys_Tar
#D-* Purpose: master un-tar command
#D-* Examples:
#D-
function IGCM_sys_Tar {
  IGCM_debug_PushStack "IGCM_sys_Tar" $@
  if ( $DEBUG_sys ) ; then
    echo "IGCM_sys_Tar :" $@
  fi
  \tar cf $@
  if [ $? -gt 0 ] ; then
    echo "IGCM_sys_Tar : erreur."
    IGCM_debug_Exit "IGCM_sys_Tar"
  fi
  IGCM_debug_PopStack "IGCM_sys_Tar"
}

#D-#==================================================
#D-function IGCM_sys_UnTar
#D-* Purpose: master un-tar command
#D-* Examples:
#D-
function IGCM_sys_UnTar {
  IGCM_debug_PushStack "IGCM_sys_UnTar" $@
  if ( $DEBUG_sys ) ; then
    echo "IGCM_sys_UnTar :" $@
  fi
  \tar xvf $1
  if [ $? -gt 0 ] ; then
    echo "IGCM_sys_UnTar : erreur."
    IGCM_debug_Exit "IGCM_sys_UnTar"
  fi
  IGCM_debug_PopStack "IGCM_sys_UnTar"
}

#D-#==================================================
#D-function IGCM_sys_Qsub
#D-* Purpose: Qsub new job
#D-* Examples:
#D-
function IGCM_sys_Qsub {
  IGCM_debug_PushStack "IGCM_sys_Qsub" $@

  typeset ERROR
  ERROR=0

  if ( $DEBUG_sys ) ; then
    echo "IGCM_sys_Qsub :" $@
  fi
  # We have to change output/error file
  [ ${#@} = 1 ] &&  REP_FOR_JOB=$LOADL_STEP_INITDIR
  [ ${#@} = 2 ] &&  REP_FOR_JOB=$2
  sed -e "s/\# \@ output *= .*/\# \@ output = ${Script_Output}/" -e "s/\# \@ error *= .*/\# \@ error = ${Script_Output}/" <$1 >${REP_FOR_JOB}/JOB_FOR_IGCM
  {
    [ ${#@} = 1 ] &&  ( cd $LOADL_STEP_INITDIR ; /usr/local/bin/llsubmit JOB_FOR_IGCM ; ERROR=$? ; cd - ; )
    [ ${#@} = 2 ] &&  ( cd $2 ; /usr/local/bin/llsubmit JOB_FOR_IGCM ; ERROR=$? ; cd - ; )
  }
  if [ ${ERROR} -gt 0 ] ; then
    echo "IGCM_sys_Qsub : erreur $@."
    IGCM_debug_Exit "IGCM_sys_Qsub"
  fi
  IGCM_sys_Rm ${REP_FOR_JOB}/JOB_FOR_IGCM
  IGCM_debug_PopStack "IGCM_sys_Qsub"

}

#D-#==================================================
#D-function IGCM_sys_QsubPost
#D-* Purpose: Qsub new job on scalaire
#D-* Examples:
#D-
function IGCM_sys_QsubPost {
  IGCM_debug_PushStack "IGCM_sys_QsubPost" $@
  if ( $DEBUG_sys ) ; then
    echo "IGCM_sys_QsubPost :" $@
  fi
  cd ${POST_DIR}
  /opt/ibmll/LoadL/full/bin/llsubmit ${libIGCM_POST}/$1.job
  cd -
  if [ $? -gt 0 ] ; then
    echo "IGCM_sys_QsubPost : erreur " $@
    IGCM_debug_Exit "IGCM_sys_QsubPost"
  fi
  IGCM_debug_PopStack "IGCM_sys_QsubPost"
}

#D-*************************
#D- File transfer functions
#D-*************************
#D-

#D-#==================================================
#D-function IGCM_sys_Rsync_out
#D-* Purpose: treat return val of rsync
#D-* Examples: IGCM_sys_Rsync_out out_RET_rsync
#D-  Error values and explanations can depend on your system version.
function IGCM_sys_Rsync_out {
  RET=$1
  if [ ! $RET ] ; then
    echo "rsync error !"
  fi

  if [ $MYLANG = "fr" ]; then
    case $RET in
    0)  return ;;
    1)  echo "Erreur de rsync ; RERR_SYNTAX : "
      echo "Erreur de syntaxe ou d'utilisation."
      return;;
    2)  echo "Erreur de rsync ; RERR_PROTOCOL : "
      echo "Incompatibilit de protocole."
      return;;
    3)  echo "Erreur de rsync ; RERR_FILESELECT 3"
      echo "Erreurs  lors  de  la  slection des fichiers d'entre sortie et"
      echo "rpertoires"
      return;;
    4)  echo "Erreur de rsync ; RERR_UNSUPPORTED"
      echo "Action demande non supporte : une tentative de manipulation de"
      echo "fichiers  64-bits  sur une plate-forme qui ne les supporte pas a"
      echo "t faite ; ou une option qui est supporte par le  client  mais"
      echo "pas par le serveur a t spcifie."
      return;;
    10) echo "Erreur de rsync ; RERR_SOCKETIO"
      echo "Erreur dans le socket d'entre sortie"
      return;;
    11) echo "Erreur de rsync ; RERR_FILEIO"
      echo "Erreur d'entre sortie fichier"
      return;;
    12) echo "Erreur de rsync ; RERR_STREAMIO"
      echo "Erreur dans flux de donne du protocole rsync"
      return;;
    13) echo "Erreur de rsync ; RERR_MESSAGEIO"
      echo "Erreur avec les diagnostics du programme"
      return;;
    14) echo "Erreur de rsync ; RERR_IPC"
      echo "Erreur dans le code IPC"
      return;;
    20) echo "Erreur de rsync ; RERR_SIGNAL"
      echo "SIGUSR1 ou SIGINT reu"
      return;;
    21) echo "Erreur de rsync ; RERR_WAITCHILD"
      echo "Une erreur retourne par waitpid()"
      return;;
    22) echo "Erreur de rsync ; RERR_MALLOC"
      echo "Erreur lors de l'allocation des tampons de mmoire de coeur"
      return;;
    23) echo ""
      echo "Erreur fichier inexistant"
      return;;
    30) echo "Erreur de rsync ; RERR_TIMEOUT"
      echo "Temps d'attente coul dans l'envoi/rception de donnes"
      return;;
    *)  echo "Erreur de rsync : code de retour de rsync inconnu :" $RET
      return;;
    esac
  elif [ $MYLANG = "en" ] ; then
    case $RET in
    0)  return;;
    1)  echo "rsync error : Syntax or usage error "
      return;;
    2)  echo "rsync error : Protocol incompatibility "
      return;;
    3)  echo "rsync error : Errors selecting input/output files, dirs"
      return;;
    4)  echo "rsync error : Requested action not supported: an attempt"
      echo "was made to manipulate 64-bit files on a platform that cannot support"
      echo "them; or an option was specified that is supported by the client and"
      echo "not by the server."
      return;;
    5)  echo "rsync error : Error starting client-server protocol"
      return;;
    10) echo "rsync error : Error in socket I/O "
      return;;
    11) echo "rsync error : Error in file I/O "
      return;;
    12) echo "rsync error : Error in rsync protocol data stream "
      return;;
    13) echo "rsync error : Errors with program diagnostics "
      return;;
    14) echo "rsync error : Error in IPC code "
      return;;
    20) echo "rsync error : Received SIGUSR1 or SIGINT "
      return;;
    21) echo "rsync error : Some error returned by waitpid() "
      return;;
    22) echo "rsync error : Error allocating core memory buffers "
      return;;
    23) echo "rsync error : Partial transfer due to error"
      return;;
    24) echo "rsync error : Partial transfer due to vanished source files"
      return;;
    30) echo "rsync error : Timeout in data send/receive "
      return;;
    *)  echo "rsync error : return code of rsync unknown :" $RET
      return;;
    esac
  else
    echo "unknown language $MYLANG."
    return
  fi
}

#D-#==================================================
#D-function IGCM_sys_Miror_libIGCM
#D-* Purpose: Mirror libIGCM PATH and lib to ulam
#D-* Examples:
#D-
function IGCM_sys_Mirror_libIGCM {
  IGCM_debug_PushStack "IGCM_sys_Mirror_libIGCM"
  if ( $DEBUG_sys ) ; then
    echo "IGCM_sys_Mirror_libIGCM"
  fi

  typeset RET

  IGCM_sys_RshPost <<-EOF
    mkdir -p ${HOME_POST}/MIRROR/${PATHlibIGCM}
EOF
  rsh ulam -n hostname > /dev/null 2>&1
  RET=$?
  if [ $RET -eq 0 ] ; then
    echo ${RSYNC} ${RSYNC_opt} -e "rsh " ${libIGCM} ulam:${HOME_POST}/MIRROR/${PATHlibIGCM} > out_rsync 2>&1
    ${RSYNC} ${RSYNC_opt} -e "rsh " ${libIGCM} ulam:${HOME_POST}/MIRROR/${PATHlibIGCM} >> out_rsync 2>&1
    RET=$?

    if [ ${RET} -gt 0 ] ; then
      echo "IGCM_sys_Mirror_libIGCM Warning : no libIGCM on ulam."
      cat out_rsync
    fi
    IGCM_sys_Rm out_rsync
  else
    echo "No POST-TREATMENT avaible because ulam is down."
  fi
  IGCM_debug_PopStack "IGCM_sys_Mirror_libIGCM"
}

#D-#==================================================
#D-function IGCM_sys_Cp
#D-* Purpose: generic cp
#D-* Examples:
#D-
function IGCM_sys_Cp {
  IGCM_debug_PushStack "IGCM_sys_Cp" $@
  if ( $DEBUG_sys ) ; then
    echo "IGCM_sys_Cp :" $@
  fi

  typeset RET

  echo cp $@ > out_rsync 2>&1
  \cp $@ >> out_rsync 2>&1
  RET=$?

  if [ ${RET} -gt 0 ] ; then
    echo "IGCM_sys_Cp : error."
    cat out_rsync
    IGCM_debug_Exit "IGCM_sys_Cp"
  else
    \rm out_rsync
  fi
  IGCM_debug_PopStack "IGCM_sys_Cp"
}

#D-#==================================================
#D-function IGCM_sys_Rm
#D-* Purpose: generic rm
#D-* Examples:
#D-
function IGCM_sys_Rm {
  IGCM_debug_PushStack "IGCM_sys_Rm" -- $@
  if ( $DEBUG_sys ) ; then
    echo "IGCM_sys_Rm :" $@
  fi

  typeset RET

  echo rm $@ > out_rsync 2>&1
  \rm $@ >> out_rsync 2>&1
  RET=$?

  if [ ${RET} -gt 0 ] ; then
    echo "IGCM_sys_Rm : error."
    cat out_rsync
    IGCM_debug_Exit "IGCM_sys_Rm"
  else
    \rm out_rsync
  fi
  IGCM_debug_PopStack "IGCM_sys_Rm"
}

#D-#==================================================
#D-function IGCM_sys_RmRunDir
#D-* Purpose: rm tmpdir (dummy function most of the time batch
#D-                      scheduler will do the job)
#D-* Examples:
#D-
function IGCM_sys_RmRunDir {
  IGCM_debug_PushStack "IGCM_sys_RmRunDir" -- $@
  if ( $DEBUG_sys ) ; then
    echo "IGCM_sys_RmRunDir :" $@
    echo "Dummy call, let the scheduler do that."
  fi
  IGCM_debug_PopStack "IGCM_sys_RmRunDir"
}

#D-#==================================================
#D-function IGCM_sys_Mv
#D-* Purpose: generic move
#D-* Examples:
#D-
function IGCM_sys_Mv {
  IGCM_debug_PushStack "IGCM_sys_Mv" $@
  if ( $DEBUG_sys ) ; then
    echo "IGCM_sys_Mv :" $@
  fi

  if [ $DRYRUN = 0 ]; then

    typeset RET

    echo mv $@ > out_rsync 2>&1
    \mv $@ >> out_rsync 2>&1
    RET=$?

    if [ ${RET} -gt 0 ] ; then
      echo "IGCM_sys_Mv : error in mv."
      cat out_rsync
      IGCM_debug_Exit "IGCM_sys_Mv"
    else
      \rm out_rsync
    fi
  else
    ( ${DEBUG_debug} ) && echo "DRYRUN mode = " $DRYRUN >> stack
  fi

  IGCM_debug_PopStack "IGCM_sys_Mv"
}

#D-#==================================================
#D-function IGCM_sys_Put_Dir
#D-* Purpose: Copy a complete directory on $(ARCHIVE)
#D-* Examples:
#D-
function IGCM_sys_Put_Dir {
  IGCM_debug_PushStack "IGCM_sys_Put_Dir" $@
  if ( $DEBUG_sys ) ; then
    echo "IGCM_sys_Put_Dir :" $@
  fi
  if [ $DRYRUN = 0 ]; then
    if [ ! -d ${1} ] ; then
      echo "WARNING : IGCM_sys_Put_Dir ${1} DOES NOT EXIST ."
      IGCM_debug_PopStack "IGCM_sys_Put_Dir"
      return
    fi

    typeset RET

    # Only if we use rsync
    #IGCM_sys_TestDirArchive $( dirname $2 )
    #
    #USUAL WAY
    rcp -r $1 ${STOREHOST}:$2 > out_rsync 2>&1
    RET=$?

    if [ ${RET} -gt 0 ] ; then
      echo "IGCM_sys_Put_Dir : error."
      cat out_rsync
      IGCM_debug_Exit "IGCM_sys_Put_Dir"
    else
      \rm out_rsync
    fi
  else
    ( ${DEBUG_debug} ) && echo "DRYRUN mode = " $DRYRUN >> stack
  fi
  IGCM_debug_PopStack "IGCM_sys_Put_Dir"
}

#D-#==================================================
#D-function IGCM_sys_Get_Dir
#D-* Purpose: Copy a complete directory from $(ARCHIVE)
#D-* Examples:
#D-
function IGCM_sys_Get_Dir {
  IGCM_debug_PushStack "IGCM_sys_Get_Dir" $@
  if ( $DEBUG_sys ) ; then
    echo "IGCM_sys_Get_Dir :" $@
  fi
  if [ $DRYRUN = 0 ]; then
    if [ ! -d ${1} ] ; then
      echo "WARNING : IGCM_sys_Get_Dir ${1} DOES NOT EXIST ."
      IGCM_debug_PopStack "IGCM_sys_Get_Dir"
      return
    fi

    typeset RET

        #USUAL WAY
    rcp -rp ${STOREHOST}:$1 $2 > out_rsync 2>&1
    RET=$?

    if [ ${RET} -gt 0 ] ; then
      echo "IGCM_sys_Get_Dir : error."
      cat out_rsync
      IGCM_debug_Exit "IGCM_sys_Get_Dir"
    fi
  else
    ( ${DEBUG_debug} ) && echo "DRYRUN mode = " $DRYRUN >> stack
  fi
  IGCM_debug_PopStack "IGCM_sys_Get_Dir"
}

#====================================================
#- Call IGCM_sys_Mirror_libIGCM now !
if ( $MirrorlibIGCM ) ; then
  IGCM_sys_Mirror_libIGCM
fi

#D-#==================================================
#D-function IGCM_sys_Put_Rest
#D-* Purpose: Put computied restarts on $(ARCHIVE).
#D-           File and target directory must exist.
#D-* Examples:
#D-
function IGCM_sys_Put_Rest {
  IGCM_debug_PushStack "IGCM_sys_Put_Rest" $@
  if ( $DEBUG_sys ) ; then
    echo "IGCM_sys_Put_Rest :" $@
  fi
  if [ $DRYRUN = 0 ]; then
    if [ ! -f ${1} ] ; then
      echo "ERROR : IGCM_sys_Put_Rest ${1} DOES NOT EXIST ."
      IGCM_debug_Exit "IGCM_sys_Put_Rest"
    fi

    typeset RET
        #
    if [ X${JobType} = XRUN ] ; then
      IGCM_sys_Chmod 444 ${1}
    fi
    #
    # Only if we use rsync
    #IGCM_sys_MkdirArchive $( dirname $2 )
    #
    #USUAL WAY
    mfput $1 $2 > out_rsync 2>&1
    RET=$?

#       #RSYNC WITH NETWORK RSH CALL
#       echo ${RSYNC} ${RSYNC_opt} --rsync-path=${REMOTE_RSYNC} -e rsh ${RUN_DIR}/$1 ${STOREHOST}:${2} > out_rsync 2>&1
#       ${RSYNC} ${RSYNC_opt} --rsync-path=${REMOTE_RSYNC} -e rsh ${RUN_DIR}/$1 ${STOREHOST}:${2} >> out_rsync 2>&1

#       #RSYNC WITH NFS USE
#       echo ${RSYNC} ${RSYNC_opt} ${RUN_DIR}/$1 ${2} > out_rsync 2>&1
#       ${RSYNC} ${RSYNC_opt} ${RUN_DIR}/$1 ${2} >> out_rsync 2>&1

#       RET=$?
#       IGCM_sys_Rsync_out $RET

#       ${libIGCM}/libIGCM_sys/IGCM_analyse_rsync_out.awk out_rsync
#       (( RET=RET+$? ))

    if [ ${RET} -gt 0 ] ; then
      echo "IGCM_sys_Put_Rest : error."
      cat out_rsync
      IGCM_debug_Exit "IGCM_sys_Put_Rest"
    else
      \rm out_rsync
    fi
  else
    ( ${DEBUG_debug} ) && echo "DRYRUN mode = " $DRYRUN >> stack
  fi
  IGCM_debug_PopStack "IGCM_sys_Put_Rest"
}

#D-#==================================================
#D-function IGCM_sys_PutBuffer_Rest
#D-* Purpose: Put computied restarts on ${SCRATCHDIR}.
#D-           File and target directory must exist.
#D-* Examples:
#D-
function IGCM_sys_PutBuffer_Rest {
  IGCM_debug_PushStack "IGCM_sys_PutBuffer_Rest" $@
  if ( $DEBUG_sys ) ; then
    echo "IGCM_sys_PutBuffer_Rest :" $@
  fi
  if [ $DRYRUN = 0 ]; then
    if [ ! -f ${1} ] ; then
      echo "ERROR : IGCM_sys_PutBuffer_Rest ${1} DOES NOT EXIST ."
      IGCM_debug_Exit "IGCM_sys_PutBuffer_Rest"
    fi

    typeset RET
    #
    if [ X${JobType} = XRUN ] ; then
      IGCM_sys_Chmod 444 ${1}
    fi

    #
    # USUAL WAY
    \cp $1 $2 > out_rsync 2>&1
    RET=$?

    if [ ${RET} -gt 0 ] ; then
      echo "IGCM_sys_PutBuffer_Rest : error."
      cat out_rsync
      IGCM_debug_Exit "IGCM_sys_PutBuffer_Rest"
    else
      \rm out_rsync
    fi
  else
    ( ${DEBUG_debug} ) && echo "DRYRUN mode = " $DRYRUN >> stack
  fi
  IGCM_debug_PopStack "IGCM_sys_PutBuffer_Rest"
}

#D-#==================================================
#D-function IGCM_sys_Put_Out
#D-* Purpose: Copy a file on $(ARCHIVE) after have chmod it in readonly
#D-* Examples:
#D-
function IGCM_sys_Put_Out {
  IGCM_debug_PushStack "IGCM_sys_Put_Out" $@
  if ( $DEBUG_sys ) ; then
    echo "IGCM_sys_Put_Out :" $@
  fi
  if [ $DRYRUN = 0 ]; then
    if [ ! -f ${1} ] ; then
      echo "WARNING : IGCM_sys_Put_Out ${1} DOES NOT EXIST ."
      IGCM_debug_PopStack "IGCM_sys_Put_Out"
      return 1
    fi

    typeset RET
        #
    if [ X${JobType} = XRUN ] ; then
      if [ X${3} = X ] ; then
        IGCM_sys_Chmod 444 ${1}
      fi
    fi
    #
    # Only if we use rsync
    #IGCM_sys_MkdirArchive $( dirname $2 )
    #
    #USUAL WAY
    mfput $1 $2 > out_rsync 2>&1
    RET=$?

#       #RSYNC WITH NETWORK RSH CALL
#       echo ${RSYNC} ${RSYNC_opt} --rsync-path=${REMOTE_RSYNC} -e rsh ${RUN_DIR}/$1 ${STOREHOST}:${2} > out_rsync 2>&1
#       ${RSYNC} ${RSYNC_opt} --rsync-path=${REMOTE_RSYNC} -e rsh ${RUN_DIR}/$1 ${STOREHOST}:${2} >> out_rsync 2>&1

#       #RSYNC WITH NFS USE
#       echo ${RSYNC} ${RSYNC_opt} ${RUN_DIR}/$1 ${2} > out_rsync 2>&1
#       ${RSYNC} ${RSYNC_opt} ${RUN_DIR}/$1 ${2} >> out_rsync 2>&1

#       RET=$?
#       IGCM_sys_Rsync_out $RET

#       ${libIGCM}/libIGCM_sys/IGCM_analyse_rsync_out.awk out_rsync
#       (( RET=RET+$? ))

    if [ ${RET} -gt 0 ] ; then
      echo "IGCM_sys_Put_Out : error."
      cat out_rsync
      IGCM_debug_Exit "IGCM_sys_Put_Out"
    else
      \rm out_rsync
    fi
    IGCM_sys_Rm out_rsync
  else
    ( ${DEBUG_debug} ) && echo "DRYRUN mode = " $DRYRUN >> stack
  fi
  IGCM_debug_PopStack "IGCM_sys_Put_Out"
  return 0
}

#D-#==================================================
#D-function IGCM_sys_PutBuffer_Out
#D-* Purpose: Copy a file on ${SCRATCHDIR} after having chmod it in readonly
#D-* Examples:
#D-
function IGCM_sys_PutBuffer_Out {
  IGCM_debug_PushStack "IGCM_sys_PutBuffer_Out" $@
  if ( $DEBUG_sys ) ; then
    echo "IGCM_sys_PutBuffer_Out :" $@
  fi
  if [ $DRYRUN = 0 ]; then
    if [ ! -f ${1} ] ; then
      echo "WARNING : IGCM_sys_PutBuffer_Out ${1} DOES NOT EXIST ."
      IGCM_debug_PopStack "IGCM_sys_PutBuffer_Out"
      return 1
    fi
    #
    IGCM_sys_Mkdir $( dirname $2 )
    #
    typeset RET

    if [ X${JobType} = XRUN ] ; then
      if [ X${3} = X ] ; then
        IGCM_sys_Chmod 444 ${1}
      fi
    fi
    #
    # USUAL WAY
    \cp $1 $2 > out_rsync 2>&1
    RET=$?

    if [ ${RET} -gt 0 ] ; then
      echo "IGCM_sys_PutBuffer_Out : error."
      cat out_rsync
      IGCM_debug_Exit "IGCM_sys_PutBuffer_Out"
    else
      \rm out_rsync
    fi
  else
    ( ${DEBUG_debug} ) && echo "DRYRUN mode = " $DRYRUN >> stack
  fi
  IGCM_debug_PopStack "IGCM_sys_PutBuffer_Out"
  return 0
}

#D-#==================================================
#D-function IGCM_sys_Get
#D-* Purpose: Get a file from ${ARCHIVE}
#D-* Examples: IGCM_sys_Get myfile /destpath/myfile_with_PREFIX
#D-            IGCM_sys_Get /l Array_contain_myfiles /destpath/
function IGCM_sys_Get {
  IGCM_debug_PushStack "IGCM_sys_Get" $@

  typeset DEST RET dm_liste ifile target

  if ( $DEBUG_sys ) ; then
    echo "IGCM_sys_Get :" $@
  fi
  if [ $DRYRUN -le 2 ]; then
    if [ X${1} = X'/l' ] ; then
      # test if the first file is present in the old computation :
      eval set +A dm_liste \${${2}}
    else
      dm_liste=${1}
    fi
    eval DEST=\${${#}}

    # test if the (first) file is present in the old computation :
    IGCM_sys_TestFileArchive ${dm_liste[0]}
    RET=$?
    if [ ${RET} -gt 0 ] ; then
      echo "IGCM_sys_Get, ERROR : regular file ${dm_liste[0]} DOES NOT EXIST ."
      IGCM_debug_Exit "IGCM_sys_Get"
      return
    fi

    # SD : dm_liste is not suited for computing job
    #      because we change filename during transfert
    #      dm_liste is better suited for post-treatment
    #dm_liste=" "
    #(( ifile=1 ))
    #while [ $ifile -lt $# ] ; do
    #    dm_liste=$( eval echo ${dm_liste} " "\${${ifile}} )
    #    (( ifile = ifile + 1 ))
    #done
    #DEST=$( eval echo \${${#}} )

    #USUAL WAY
    mfget ${dm_liste[*]} ${DEST} > out_rsync 2>&1
    RET=$?

#       #RSYNC WITH NETWORK RSH CALL
#       echo ${RSYNC} ${RSYNC_opt} --rsync-path=${REMOTE_RSYNC} -e rsh ${STOREHOST}:"${dm_liste}" ${STOREHOST}:${RUN_DIR}/${DEST} > out_rsync 2>&1
#       ${RSYNC} ${RSYNC_opt} --rsync-path=${REMOTE_RSYNC} -e rsh ${STOREHOST}:"${dm_liste}" ${STOREHOST}:${RUN_DIR}/${DEST} >> out_rsync 2>&1

#       #RSYNC WITH NFS USE
#       echo ${RSYNC} ${RSYNC_opt} ${dm_liste} ${RUN_DIR}/${DEST} > out_rsync 2>&1
#       ${RSYNC} ${RSYNC_opt} ${dm_liste} ${RUN_DIR}/${DEST} >> out_rsync 2>&1

#       RET=$?
#       IGCM_sys_Rsync_out $RET

#       ${libIGCM}/libIGCM_sys/IGCM_analyse_rsync_out.awk out_rsync
#       (( RET=RET+$? ))

    if [ ${RET} -gt 0 ] ; then
      echo "IGCM_sys_Get : copy error."
      cat out_rsync
      IGCM_debug_Exit "IGCM_sys_Get"
    else
      \rm out_rsync
    fi
  else
    ( ${DEBUG_debug} ) && echo "DRYRUN mode = " $DRYRUN >> stack
  fi
  IGCM_debug_PopStack "IGCM_sys_Get"
}

#D-#==================================================
#D-function IGCM_sys_GetBuffer
#D-* Purpose: Get a file from ${SCRATCHDIR}
#D-* Examples: IGCM_sys_GetBuffer myfile /destpath/myfile_with_PREFIX
#D-            IGCM_sys_GetBuffer /l Array_contain_myfiles /destpath/
function IGCM_sys_GetBuffer {
  IGCM_debug_PushStack "IGCM_sys_GetBuffer" $@

  typeset DEST RET buf_liste ifile target file_work

  if ( $DEBUG_sys ) ; then
    echo "IGCM_sys_GetBuffer :" $@
  fi
  if [ $DRYRUN -le 2 ]; then
    if [ X${1} = X'/l' ] ; then
      # test if the first file is present in the old computation :
      eval set +A buf_liste \${${2}}
    else
      eval set +A buf_liste ${1}
    fi
    eval DEST=\${${#}}

    #USUAL WAY
    if [ X${1} = X'/l' ] ; then
      (( RET=0 ))
      for target in ${buf_liste[*]} ; do
        local_file=$( basename ${target} )
        \cp ${target} ${DEST}/${local_file} >> out_rsync 2>&1
        (( RET = RET + $? ))
      done
    else
      \cp ${buf_liste} ${DEST} >> out_rsync 2>&1
      RET=$?
    fi

    if [ ${RET} -gt 0 ] ; then
      echo "IGCM_sys_GetBuffer : copy error."
      cat out_rsync
      IGCM_debug_Exit "IGCM_sys_GetBuffer"
    else
      \rm out_rsync
    fi
  else
    ( ${DEBUG_debug} ) && echo "DRYRUN mode = " $DRYRUN >> stack
  fi
  IGCM_debug_PopStack "IGCM_sys_GetBuffer"
}

############################################################## A FINIR !!

#D-#==================================================
#D-function IGCM_sys_GetDate_FichWork
#D-* Purpose: donne la date filesys d'un fichier sur le filesystem WORK
#D-* Examples:
#D-
function IGCM_sys_GetDate_FichWork {
  IGCM_debug_PushStack "IGCM_sys_FichWork" $@
  if ( $DEBUG_sys ) ; then
    echo "IGCM_sys_GetDate_FichWork :" $@
  fi
  typeset dateF
  set +A dateF -- $( rsh ${MASTER} -n /opt/freeware/bin/ls -l --full-time --time-style='+%Y%m%d%H%M%S' ${1} )
  eval ${2}=${dateF[5]}

  # donne la date filesys d'un fichier sur la machine work
  IGCM_debug_PopStack "IGCM_sys_FichWork"
}

#D-#==================================================
#D-function IGCM_sys_GetDate_FichArchive
#D-* Purpose: donne la date filesys d'un fichier sur le filesystem ARCHIVE
#D-* Examples:
#D-
function IGCM_sys_GetDate_FichArchive {
  IGCM_debug_PushStack "IGCM_sys_FichArchive" $@
  if ( $DEBUG_sys ) ; then
    echo "IGCM_sys_GetDate_FichArchive :" $@
  fi
  typeset dateF
  set +A dateF -- $( rsh ${STOREHOST} -n ls -l --full-time --time-style='+%Y%m%d%H%M%S' ${1} )
  eval ${2}=${dateF[5]}

  IGCM_debug_PopStack "IGCM_sys_FichArchive"
}

##############################################################
# REBUILD OPERATOR

function IGCM_sys_rebuild {
  IGCM_debug_PushStack "IGCM_sys_rebuild" -- $@
  if ( $DEBUG_sys ) ; then
    echo "IGCM_sys_rebuild :" $@
  fi
  /homegpfs/rech/psl/rpsl035/bin/rebuild -f -o $@
  if [ $? -gt 0 ] ; then
    echo "IGCM_sys_rebuild : erreur ${@}."
    IGCM_debug_Exit "rebuild"
  fi

  IGCM_debug_PopStack "IGCM_sys_rebuild"
}

function IGCM_sys_rebuild_station {
  IGCM_debug_PushStack "IGCM_sys_rebuild_station" -- $@
  typeset i list_opt file_in file_out prefix_invert list_invert
  if ( $DEBUG_sys ) ; then
    echo "IGCM_sys_rebuild_station :" $@
  fi
  list_opt=$@

  # Invert Axis : t,x -> x,t 
  #               t,pres,x -> x,t,pres
  # So that we can concatenate along x
  i=0
  for file_in in ${list_opt} ; do
    (( i = i + 1))
    [ ${i} = 1 ] && file_out=${file_in} && continue
    prefix_invert=$( basename ${file_in} .nc )
    IGCM_sys_ncpdq -a x,time_counter -a x,time_counter,presnivs ${file_in} ${prefix_invert}_xt.nc
    list_invert[${#list_invert[*]}]=${prefix_invert}_xt.nc
  done

  # Concatenate
  IGCM_sys_ncrcat ${list_invert[*]} histstn_xt.nc

  # Re-ivert file
  IGCM_sys_ncpdq -a time_counter,x -a time_counter,presnivs,x histstn_xt.nc ${file_out}

  # Station re-ordering is too expansive to be run within libICGM
  # This is due to (ncpdq - nrcat - ncpdq) I/O sequence.
  # This re-ordering must be done "in memory" by the cmorization process
  # Anyway this is the best sequence using (ncpdq - nrcat - ncpdq)
  # BEGIN reordering

  # Only LMDZ text output contains the exact ordering of the station.
  # We isolate this in the code below:
  #  0  38  -157.5000000000000  70.98591549295774
  #  0  54  27.49999999999999   67.18309859154928
  #  0  56  -62.50000000000001  82.39436619718309
  #  0  79  12.49999999999999   78.59154929577466
  #  0  116 -165.0000000000000  76.05633802816901
  #  0  117 130.0000000000000   70.98591549295774
  #  0  118 110.0000000000000   87.46478873239437
  #  1  40  4.999999999999995   51.97183098591550
#  typeset iStation iProc list_opt file_in file_out prefix_invert
#  typeset -Z4 j4
#  typeset -Z3 j3

#  unset list_opt
#  set +A list_opt $@

  # Filename after rebuild
#  file_out=${list_opt[0]}
  # Prefix of output files
#  prefix_invert=$( basename ${file_out} .nc )
  # Number of procs
#  num_proc=$( grep -i mpi_size ${PREFIX}_${Exe_Output} | wc -l )

#  iProc=0
#  while [ ${iProc} -lt ${num_proc} ] ; do
    # Array containing Station as a number
#    unset proc_stn
#    set +A proc_stn $( grep "iophy_mpi rank ip lon lat  $iProc" ${PREFIX}_${Exe_Output} | sed -e "s/iophy_mpi rank ip lon lat //g" | awk ' {print $2}' )
    # Number of stations produced by processor proc
#    stationLast=${#proc_stn[*]}
    # Proc number on 4 digits
#    j4=${iProc}
    # Init
#    iStation=0
#    while [ ${iStation} -lt ${stationLast} ] ; do
      # Station number on 3 digits
#      j3=${proc_stn[${iStation}]}
      # Extract station
      # Invert Axis : t,x -> x,t 
      #               t,pres,x -> x,t,pres
      # So that we can concatenate along x
#      IGCM_sys_ncpdq -a x,time_counter -a x,time_counter,presnivs -d x,$iStation,$iStation ${prefix_invert}_${j4}.nc ${prefix_invert}_stn_${j3}.nc
#      (( iStation = iStation + 1 ))
#    done
#    (( iProc = iProc + 1 ))
#  done

  # Concatenate all station along x
#  IGCM_sys_ncrcat ${prefix_invert}_stn_???.nc ${prefix_invert}_xt.nc

  # Re-invert file
#  IGCM_sys_ncpdq -a time_counter,x -a time_counter,presnivs,x ${prefix_invert}_xt.nc ${file_out}

  # END reordering

  IGCM_debug_PopStack "IGCM_sys_rebuild_station"
}

############################################################
# Activate Running Environnment Variables

function IGCM_sys_activ_variables {
  IGCM_debug_PushStack "IGCM_sys_activ_variables"
  if ( $DEBUG_sys ) ; then
    echo "IGCM_sys_activ_variables "
  fi

# --------------------------------------------------------------------
#D- OpenMP thread memory specifications : only one time
# --------------------------------------------------------------------
echo $XLSMPOPTS | grep stack=524288000 1>/dev/null 2>&1 || export XLSMPOPTS=$XLSMPOPTS:stack=524288000

# --------------------------------------------------------------------
#D- Other specifications
# --------------------------------------------------------------------

  IGCM_debug_PopStack "IGCM_sys_activ_variables"

}

############################################################
# Desactivate Running Environnment Variables

function IGCM_sys_desactiv_variables {
  IGCM_debug_PushStack "IGCM_sys_desactiv_variables"
  if ( $DEBUG_sys ) ; then
    echo "IGCM_sys_desactiv_variables "
  fi
# --------------------------------------------------------------------
#D- MPI specifications
# --------------------------------------------------------------------

# --------------------------------------------------------------------
#D- Other specifications
# --------------------------------------------------------------------

  IGCM_debug_PopStack "IGCM_sys_desactiv_variables"

}

############################################################
# Build MPI/OMP scripts
function IGCM_sys_build_execution_scripts
{
  IGCM_debug_PushStack "IGCM_sys_build_execution_scripts" $@
  if ( $DEBUG_sys ) ; then
    echo "IGCM_sys_build_execution_scripts " $@
  fi

  typeset NbNodes_Job NbProc_Job comp_proc_mpi_loc comp_proc_omp_loc mpi_count

  if [ ! -f ${SUBMIT_DIR}/Job_${config_UserChoices_JobName} ]  ; then
    IGCM_debug_Exit "IGCM_sys_vargas build_execution_scripts : Job_${config_UserChoices_JobName} don't exist in SUBMIT_DIR : ${SUBMIT_DIR} "
  fi

  if ( ${OK_PARA_MPMD} ) ; then

    if [ -f run_file ] ; then
      IGCM_sys_Rm -f run_file
    fi
    touch run_file

    if ( ${OK_PARA_OMP} ) ; then
# NEW : 2 Noeuds
# @ task_geometry={(0)(1,2,3)}
# Nombre de processus demandes
      echo "Job_${config_UserChoices_JobName} includes task_geometry = \c"
      cat ${SUBMIT_DIR}/Job_${config_UserChoices_JobName} | grep "task_geometry" | sed -e "s/[^(]*([^(]*/(/g"
      echo "Job_${config_UserChoices_JobName} includes task_geometry  with NbNodes = \c"
      cat ${SUBMIT_DIR}/Job_${config_UserChoices_JobName} | grep "task_geometry" | sed -e "s/[^(]*([^(]*/(/g" | wc -c
      NbNodes_Job=$(( $( cat ${SUBMIT_DIR}/Job_${config_UserChoices_JobName} | grep "task_geometry" | sed -e "s/[^(]*([^(]*/(/g" | wc -c ) - 1 ))

      if [ ${NbNodes_Job} -eq 0 ] ; then
        IGCM_debug_Print 2 "Error in Job_${config_UserChoices_JobName} ressources : no task_geometry defined with OpenMP run."
        exit 1
      fi
    else

# OLD :
# @ total_tasks = 3
# @ environment = "BATCH_NUM_PROC_TOT=3"
# Pas d OpenMP
# @ resources = ConsumableCpus(1)

      echo "Job_${config_UserChoices_JobName} includes total_tasks = \c"
      cat ${SUBMIT_DIR}/Job_${config_UserChoices_JobName} | grep "total_tasks" | sed -e "s/.*total_tasks = //"
      NbProc_Job=$( cat ${SUBMIT_DIR}/Job_${config_UserChoices_JobName} | grep "total_tasks" | sed -e "s/.*total_tasks = //" )
      NbProc_Job=${NbProc_Job:=0}
      if [ ${NbProc_Job} -eq 0 ] ; then
        IGCM_debug_Print 2 "Error in Job_${config_UserChoices_JobName} ressources : no total_tasks defined with MPI only run."
        exit 1
      fi
      if ( $( egrep '^# *@ *resources *= *ConsumableCpus\(1\)' ${SUBMIT_DIR}/Job_${config_UserChoices_JobName} >/dev/null 2>&1 ) ) ; then
        IGCM_debug_Print 2 "ressources =  ConsumableCpus(1) line found into Job_${config_UserChoices_JobName}"
      else
        IGCM_debug_Print 2 "Error in Job_${config_UserChoices_JobName} ressources = line not found. Job should include resources = ConsumableCpus(1) "
        exit 1
      fi
    fi

# run_file construction

# Then first loop on the components for the coupler ie oasis

### the coupler ie oasis must be the first one
    for comp in ${config_ListOfComponents[*]} ; do

      eval ExeNameIn=\${config_Executable_${comp}[0]}
      eval ExeNameOut=\${config_Executable_${comp}[1]}

      # for CPL component only
      if [ "X${comp}" = "XCPL" ] ; then

        eval comp_proc_mpi_loc=\${${comp}_PROC_MPI}
        eval comp_proc_omp_loc=\${${comp}_PROC_OMP}

        if ( ${OK_PARA_MPI} ) ; then

          (( mpi_count = 1 ))
          until [ ${mpi_count} -gt ${comp_proc_mpi_loc} ] ; do
            if ( ${OK_PARA_OMP} ) ; then
              echo "env OMP_NUM_THREADS=${comp_proc_omp_loc} ./${ExeNameOut} " >> run_file
            else
              echo "./${ExeNameOut}" >> run_file
            fi
            (( mpi_count = mpi_count + 1 ))
          done
        else
          if ( ${OK_PARA_OMP} ) ; then
            echo "env OMP_NUM_THREADS=${comp_proc_omp_loc} ./${ExeNameOut} " >> run_file
          else
            echo "./${ExeNameOut} " >> run_file
          fi
        fi
      fi
    done

# Then second loop on the components

    for comp in ${config_ListOfComponents[*]} ; do

      eval ExeNameIn=\${config_Executable_${comp}[0]}
      eval ExeNameOut=\${config_Executable_${comp}[1]}

      # Only if we really have an executable for the component and not the coupler ie oasis:
      if ( [ "X${ExeNameOut}" != X\"\" ] && [ "X${comp}" != "XCPL" ] ) ; then

        eval comp_proc_mpi_loc=\${${comp}_PROC_MPI}
        eval comp_proc_omp_loc=\${${comp}_PROC_OMP}

        if ( ${OK_PARA_MPI} ) ; then

          (( mpi_count = 1 ))
          until [ ${mpi_count} -gt ${comp_proc_mpi_loc} ] ; do
            if ( ${OK_PARA_OMP} ) ; then
              echo "env OMP_NUM_THREADS=${comp_proc_omp_loc} ./${ExeNameOut} " >> run_file
            else
              echo "./${ExeNameOut}" >> run_file
            fi
            (( mpi_count = mpi_count + 1 ))
          done
        else
          if ( ${OK_PARA_OMP} ) ; then
            echo "env OMP_NUM_THREADS=${comp_proc_omp_loc} ./${ExeNameOut} " >> run_file
          else
                    # to be tested : no MPI only OpenMP into MPMD mode
            echo "./${ExeNameOut} " >> run_file
          fi
        fi
      fi
    done

    EXECUTION="${HOST_MPIRUN_COMMAND} -pgmmodel mpmd -cmdfile ./run_file"

    IGCM_sys_Chmod u+x run_file
    if ( $DEBUG_sys ) ; then
      echo "run_file contains : "
      cat run_file
    fi

  else # Only one executable. launch it.

    for comp in ${config_ListOfComponents[*]} ; do

      eval ExeNameIn=\${config_Executable_${comp}[0]}
      eval ExeNameOut=\${config_Executable_${comp}[1]}

      if ( [ "X${ExeNameOut}" != X\"\" ] && [ "X${ExeNameOut}" != "Xinca.dat" ] ) ; then
        EXECUTION="time ./${ExeNameOut}"
      fi

    done

  fi

  IGCM_debug_Print 1 "sys vargas : execution command is "
  IGCM_debug_Print 1 "$EXECUTION"

  IGCM_debug_PopStack "IGCM_sys_build_execution_scripts"
}

function IGCM_sys_build_run_file {

  IGCM_debug_Print 3 " dummy function : IGCM_sys_build_run_file "

}

############################################################
# Check of space available on temporary filesytems
function IGCM_sys_check_quota {
  IGCM_debug_PushStack "IGCM_sys_check_quota" 
  if ( $DEBUG_sys ) ; then
    echo "IGCM_sys_check_quota "
  fi
  # Limit of quota (in %)
  limit_quota=90
  # Check of the volume
  volume_quota=$(quota_u -w | grep 'Quota soft' | awk '{print $5}')
  if [ ! X${volume_quota} = X ] ; then
    quota_volume=${volume_quota%%\%}
#    echo $quota_volume
    if [ ${quota_volume} -ge ${limit_quota} ] ; then
      IGCM_debug_Print 1 "Please, check your quota of volume on workgpfs"
      IGCM_debug_Print 1 "${quota_volume}% of your quota is used"
      IGCM_debug_Print 1 "Use the quota_u -w command to check"
      IGCM_debug_Print 1 "You must have more than 10% available to run"
      IGCM_debug_Exit "Not enough space to run ! STOP HERE"
      IGCM_debug_Verif_Exit
    fi

  fi
  IGCM_debug_PopStack "IGCM_sys_check_quota"
}
